'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.datePaged = datePaged;
exports.addSelfLink = addSelfLink;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _qs = require('qs');

var _qs2 = _interopRequireDefault(_qs);

var _searchProvidersSearchProvider = require('./search-providers/search-provider');

function datePaged(opt) {
  var hasPrev = existsPreviousLink(opt);
  var hasNext = existsNextLink(opt);

  removeRemainingItem(opt, hasPrev);

  if (hasPrev) {
    buildLink(opt, _searchProvidersSearchProvider.DIRECTION.BACKWARD);
  }

  if (hasNext) {
    buildLink(opt, _searchProvidersSearchProvider.DIRECTION.FORWARD);
  }

  if (!opt.data._links) {
    opt.data._links = {};
  }

  opt.data._links.self = { href: opt.root };

  if (opt.query) {
    opt.data._links.self.href += '?' + _qs2['default'].stringify(opt.query);
  }
}

function removeRemainingItem(opt, hasPrev) {
  /* eslint consistent-return: 0 */
  if (!opt.data.list.length || opt.data.list.length <= opt.limit) return;
  if (opt.query.dir !== _searchProvidersSearchProvider.DIRECTION.BACKWARD) return opt.data.list.pop();
  if (hasPrev) return opt.data.list.shift();
}

function existsNextLink(opt) {
  if (!opt.limit) return true;
  return opt.data.list.length > opt.limit || opt.query.dir === _searchProvidersSearchProvider.DIRECTION.BACKWARD && opt.data.list.length === opt.limit;
}

function existsPreviousLink(opt) {
  if (!opt.query.offset_id) return false;
  if (opt.data.list.length === 1) return true;
  return opt.query.offset_id !== opt.data.list[0][opt.idField] && (opt.data.list.length > opt.limit || opt.query.dir === _searchProvidersSearchProvider.DIRECTION.FORWARD);
}

function buildLink(opt, direction) {
  var isNext = direction === _searchProvidersSearchProvider.DIRECTION.FORWARD;
  var index = isNext ? opt.data.list.length - 1 : 0;
  var item = opt.data.list[index];
  var q = _extends({}, opt.query);

  if (!opt.data._links) opt.data._links = {};

  var offset = getPropertyValue(opt.sortFieldName.replace(/^-/, ''), item);

  if (offset) {
    if (isDate(offset)) q.offset_date = offset.toISOString();else q.offset_sort = offset.toString();
  }

  var offsetId = getPropertyValue(opt.idField, item);
  if (offsetId) q.offset_id = offsetId.toString();

  q.dir = direction;

  opt.data._links[isNext ? 'next' : 'previous'] = { href: opt.root + '?' + _qs2['default'].stringify(q) };
}

function addSelfLink(data, id, root) {
  data._links = data._links || {};

  var href = root;

  if (data[id]) {
    href += '/' + data[id];
  }

  data._links.self = { href: href };
}

function getPropertyValue(propName, item) {
  if (propName.column && propName.json) {
    return item[propName.json];
  }

  var names = propName.split('.');
  var obj = item;

  names.forEach(function (name) {
    return obj = obj[name];
  });

  return obj;
}

function isDate(date) {
  return typeof date.getMonth === 'function';
}